@@ -119,7 +119,6 @@ local function searchFieldByLocalID(source, key, pushResult)
             end
         end
     end
-    
 
     local hasMarkDoc = {}
     for _, src in ipairs(fields) do
@@ -177,7 +176,7 @@ end
 
 local searchFieldSwitch = util.switch()
     : case 'table'
-    : call(function (suri, source, key, pushResult)
+    : call(function (_suri, source, key, pushResult)
         local hasFiled = false
         for _, field in ipairs(source) do
             if field.type == 'tablefield'
@@ -247,7 +246,7 @@ local searchFieldSwitch = util.switch()
     end)
     : case 'string'
     : case 'doc.type.string'
-    : call(function (suri, source, key, pushResult)
+    : call(function (suri, _source, key, pushResult)
         -- change to `string: stringlib` ?
         local stringlib = vm.getGlobal('type', 'stringlib')
         if stringlib then
@@ -270,7 +269,7 @@ local searchFieldSwitch = util.switch()
         end
     end)
     : case 'doc.type.table'
-    : call(function (suri, source, key, pushResult)
+    : call(function (_suri, source, key, pushResult)
         if type(key) == 'string' and key:find(vm.ID_SPLITE) then
             return
         end
@@ -631,15 +630,17 @@ local function matchCall(source)
         newNode.originNode = myNode
         vm.setNode(source, newNode, true)
         if call.args then
-            -- clear node caches of args to allow recomputation with the type narrowed call
+            -- clear existing node caches of args to allow recomputation with the type narrowed call
             for _, arg in ipairs(call.args) do
-                vm.setNode(arg, vm.createNode(), true)
+                if vm.getNode(arg) then
+                    vm.setNode(arg, vm.createNode(), true)
+                end
             end
             for n in newNode:eachObject() do
                 if n.type == 'function'
                 or n.type == 'doc.type.function' then
                     for i, arg in ipairs(call.args) do
-                        if n.args[i] then
+                        if vm.getNode(arg) and n.args[i] then
                             vm.setNode(arg, vm.compileNode(n.args[i]))
                         end
                     end
@@ -961,6 +962,26 @@ local function compileCallArgNode(arg, call, callNode, fixIndex, myIndex)
     local function dealDocFunc(n)
         local myEvent
         if n.args[eventIndex] then
+            if eventMap and myIndex > eventIndex then
+                -- if call param has literal types, then also check if function def param has literal types
+                -- 1. has no literal values => not enough info, thus allowed by default
+                -- 2. has literal values and >= 1 matches call param's literal types => allowed
+                -- 3. has literal values but none matches call param's literal types => filtered
+                local myEventMap = vm.getLiterals(n.args[eventIndex])
+                if myEventMap then
+                    local found = false
+                    for k in pairs(eventMap) do
+                        if myEventMap[k] then
+                            -- there is a matching literal
+                            found = true
+                            break
+                        end
+                    end
+                    if not found then
+                        return
+                    end
+                end
+            end
             local argNode = vm.compileNode(n.args[eventIndex])
             myEvent = argNode:get(1)
         end
@@ -1117,26 +1138,56 @@ local function compileFunctionParam(func, source)
     end
     ---@cast aindex integer
 
-    -- local call ---@type fun(f: fun(x: number));call(function (x) end) --> x -> number
     local funcNode = vm.compileNode(func)
-    local found = false
-    for n in funcNode:eachObject() do
-        if n.type == 'doc.type.function' and n.args[aindex] then
-            local argNode = vm.compileNode(n.args[aindex])
-            for an in argNode:eachObject() do
-                if an.type ~= 'doc.generic.name' then
-                    vm.setNode(source, an)
+    if func.parent.type == 'callargs' then
+        -- local call ---@type fun(f: fun(x: number));call(function (x) end) --> x -> number
+        for n in funcNode:eachObject() do
+            if n.type == 'doc.type.function' and n.args[aindex] then
+                local argNode = vm.compileNode(n.args[aindex])
+                for an in argNode:eachObject() do
+                    if an.type ~= 'doc.generic.name' then
+                        vm.setNode(source, an)
+                    end
                 end
-            end
-            -- NOTE: keep existing behavior for local call which only set type based on the 1st match
-            if func.parent.type == 'callargs' then
+                -- NOTE: keep existing behavior for function as argument which only set type based on the 1st match
                 return true
             end
-            found = true
         end
-    end
-    if found then
-        return true
+    else
+        -- function declaration: use info from all `fun()`, also from the base function when overriding
+        --[[
+            ---@type fun(x: string)|fun(x: number)
+            local function f1(x) end --> x -> string|number
+
+            ---@overload fun(x: string)
+            ---@overload fun(x: number)
+            local function f2(x) end --> x -> string|number
+
+            ---@class A
+            local A = {}
+            ---@param x number
+            function A:f(x) end --> x -> number
+            ---@type A
+            local a = {}
+            function a:f(x) end --> x -> number
+        ]]
+        local found = false
+        for n in funcNode:eachObject() do
+            if (n.type == 'doc.type.function' or n.type == 'function')
+            and n.args[aindex] and n.args[aindex] ~= source
+            then
+                local argNode = vm.compileNode(n.args[aindex])
+                for an in argNode:eachObject() do
+                    if an.type ~= 'doc.generic.name' then
+                        vm.setNode(source, an)
+                    end
+                end
+                found = true
+            end
+        end
+        if found then
+            return true
+        end
     end
 
     local derviationParam = config.get(guide.getUri(func), 'Lua.type.inferParamType')
@@ -1158,7 +1209,56 @@ local function compileFunctionParam(func, source)
             end
             ::continue::
         end
-        return found
+        if found then
+            return true
+        end
+        -- infer local callback function param type
+        --[[
+            ---@param callback fun(a: integer)
+            function register(callback) end
+
+            local function callback(a) end  --> a: integer
+            register(callback)
+        ]]
+        for _, ref in ipairs(refs) do
+            if ref.parent.type ~= 'callargs' then
+                goto continue
+            end
+            -- the parent function is a variable used as callback param, find the callback arg index first
+            local call = ref.parent.parent
+            local cbIndex
+            for i, arg in ipairs(call.args) do
+                if arg == ref then
+                    cbIndex = i
+                    break
+                end
+            end
+            ---@cast cbIndex integer
+
+            -- simulate a completion at `cbIndex` to infer this callback function type
+            ---@diagnostic disable-next-line: missing-fields
+            local node = vm.compileCallArg({ type = 'dummyarg', uri = guide.getUri(call) }, call, cbIndex)
+            if not node then
+                goto continue
+            end
+            for n in node:eachObject() do
+                -- check if the inferred function has arg at `aindex`
+                if n.type == 'doc.type.function' and n.args and n.args[aindex] then
+                    -- use type info on this `aindex` arg
+                    local argNode = vm.compileNode(n.args[aindex])
+                    for an in argNode:eachObject() do
+                        if an.type ~= 'doc.generic.name' then
+                            vm.setNode(source, an)
+                            found = true
+                        end
+                    end
+                end
+            end
+            ::continue::
+        end
+        if found then
+            return true
+        end
     end
 
     do
@@ -1171,22 +1271,27 @@ local function compileFunctionParam(func, source)
             for _, set in ipairs(classDef:getSets(suri)) do
                 if set.type == 'doc.class' and set.extends then
                     for _, ext in ipairs(set.extends) do
+                        if not ext[1] then
+                            goto continue
+                        end
                         local extClass = vm.getGlobal('type', ext[1])
-                        if extClass then
-                            vm.getClassFields(suri, extClass, key, function (field, isMark)
-                                for n in vm.compileNode(field):eachObject() do
-                                    if n.type == 'function' and n.args[aindex] then
-                                        local argNode = vm.compileNode(n.args[aindex])
-                                        for an in argNode:eachObject() do
-                                            if an.type ~= 'doc.generic.name' then
-                                                vm.setNode(source, an)
-                                                found = true
-                                            end
+                        if not extClass then
+                            goto continue
+                        end
+                        vm.getClassFields(suri, extClass, key, function (field, _isMark)
+                            for n in vm.compileNode(field):eachObject() do
+                                if n.type == 'function' and n.args[aindex] then
+                                    local argNode = vm.compileNode(n.args[aindex])
+                                    for an in argNode:eachObject() do
+                                        if an.type ~= 'doc.generic.name' then
+                                            vm.setNode(source, an)
+                                            found = true
                                         end
                                     end
                                 end
-                            end)
-                        end
+                            end
+                        end)
+                        ::continue::
                     end
                 end
             end
@@ -1637,7 +1742,7 @@ local compilerSwitch = util.switch()
                             end
                             local hasGeneric
                             if sign then
-                                guide.eachSourceType(rtn, 'doc.generic.name', function (src)
+                                guide.eachSourceType(rtn, 'doc.generic.name', function (_src)
                                     hasGeneric = true
                                 end)
                             end
@@ -1833,6 +1938,10 @@ local compilerSwitch = util.switch()
     end)
     : case 'call'
     : call(function (source)
+        -- ignore rawset
+        if source.node.special == 'rawset' then
+            return
+        end
         local node = getReturn(source.node, 1, source.args)
         if not node then
             return
